;       M6P0380.ASM
;-----------------------------------------------------------------------------
;       Programm 0380h: Blink LED als NI-Task 
;-----------------------------------------------------------------------------
;
;       Zur Funktion des Programms:
;
;       Mit Parameter 0 (Command/Status) des Programms wird das Programm und 
;       damit das Blinken der LED gestartet (Parameter 0 = 1 startet das 
;       Blinken. Danach wird Parameter 0 vom Programm selbst = 2 gesetzt und 
;       zeigt dadurch an, dass es laeuft. 
;       Wenn Parameter 0 wieder = 0 gesetzt wird, stopt das Programm und das
;       Blinken hoert auf. In diesem Zustand kann durch Setzen von Parameter 
;       1 = 0 die LED aus- bzw. = 1 eingeschaltet werden. Den gleichen Effekt 
;       kann man auch durch Aufruf der Prozeduren 2 bzw. 3 dieses Programms 
;       erreichen.
;
;       Zur Programmierung:
;
;       Dieses Programm ist im IDEAL Mode in Borland Turbo-Assembler 
;       (Tiny Model) geschrieben. Es kann mit M6P0380.BAT uebersetzt und 
;       gelinkt werden. Das Ergebnis ist eine .LIB Datei.
;
;       Das Programm ist moeglichst einfach gehalten. Es soll die 
;       Programmierung einer einfachen NI-Task auf der Multi-COM Karte 
;       und deren Installierung zeigen. Zusaetzlich sind die Prozeduren 2 
;       und 3 vorgesehen, die mit dem eigentlichen Programm nichts zu tun 
;       haben, aber es sollte hier auch gezeigt werden, wie eine vom PC 
;       oder von einer anderen Task aufrufbare Prozedur aussehen kann.
;
;       Die Auto-Init Prozedur (Prozedur 1) sorgt hier fuer eine 
;       Initialisierung der Parameter. Das koennte man sich auch sparen. 
;       Beachten Sie aber, dass in der PDT alle Prozeduren fortlaufend 
;       durchnummeriert sind und keine Adresse fehlen darf (wenn noch 
;       weitere Prozeduren folgen). Wenn also die Auto-Init Prozedur nicht 
;       benoetigt wird, muss in der PDT trotzdem eine gueltige Adresse stehen. 
;       Es ist ratsam, dann auch eine Auto-Init Prozedur vorzusehen, die aber
;       nichts macht, sie besteht also nur aus "retf". 
;       Die Auto-Init Prozedur kann wie alle anderen Prozeduren auch vom PC
;       oder von einer anderen Task auf der Karte aufgerufen werden. Sie unter-
;       scheidet sich nicht von anderen Prozeduren, ausser dass sie als Teil
;       des Installierungsvorgangs zum Schluss der Installierung des Programms 
;       unter einer Task automatisch aufgerufen werden kann. 
;
;       Beachten Sie, dass die in der PDT angegebenen Adressen relative
;       Adressen sind. Nach dem Laden des Programms auf die Karte beim 
;       Installieren des Programms unter einer Task erfolgt eine Anpassung 
;       dieser Adressen (Relozierung).
;
;       Diese Programm hat keinen Datenbereich, der Parameterbereich wird 
;       lokal vom Programm selbst reserviert. In diesem Fall ist zu beachten, 
;       dass vor dem Parameterbereich immer Platz fuer die TDT frei gehalten 
;       werden muss (s.u.).
;       
;       Zur Installierung:
;
;       Das Programm kann mit SNW und M6P0380.INS unter 
;       Task 10h installiert und sofort gestartet werden. Danach sollte die 
;       LED auf der Karte blinken.

;       Das Programm hat keinen Datenbereich, der Parameterbereich wird 
;       lokal vom Programm selbst reserviert.
;
;       Beim Installieren wird auch der Task-Typ (0 = NI-Task) und das 
;       Programmformat angegeben.
;
;       Beispiel fuer Installierung dieses Programms (Datei: M6P0380.INS)
;       unter Task 10h und anschliessender Aktivierung der Task und Start:
;
;       M6INST M6P0380.LIB 0380 0010 00 000000 00000000 ; Installierung
;       M6CMD 41 02 10 00                               ; Aktivieren der Task 10h
;       M6PAR 10 00 01                                  ; Par 0 = 1: Start
;
;-----------------------------------------------------------------------------
;
;       Version 1.B. (20.08.1998 15:30): 
;
;-----------------------------------------------------------------------------
	TITLE   M6P0380
;-----------------------------------------------------------------------------

	MODEL TINY
	.186
	.CODE
	IDEAL

;-----------------------------------------------------------------------------

VERS0380        equ     '1'             ; Programm-Version ASCII: 1234
REV0380         equ     'B'             ; Programm-Revision ASCII: ABCD
TYPPDT          equ     1               ; Typ der PDT (Konstante)
DEFLPDT         equ     48              ; Laenge des Vorspanns der PDT (Konst.)
DEFLTDT         equ     36              ; Laenge der TDT (Konstante)
LED_ON          equ     400h+41*4       ; Sytem-Subroutine 41
LED_OFF         equ     400h+42*4       ; Sytem-Subroutine 42

;-----------------------------------------------------------------------------
;       M6P0380: Programm-Deskriptor-Tabelle (PDT) 
;-----------------------------------------------------------------------------
M6P0380:
	DB      TYPPDT          ; 0     ; Typ der PDT
	DB      DEFLPDT         ; 1     ; Laenge des Vorspanns der PDT
	DW      (M6P0380E - M6P0380 - DEFLPDT)/4  ; Anzahl Prozeduren
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DW      00380h          ; 4     ; Programm-Nr. 
	DB      VERS0380        ; 6     ; Programm-Version 
	DB      REV0380         ; 7     ; Programm-Revision
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DB      4               ; 8     ; Prozessor-Typ (1=186/V20,4=486)
	DB      0               ; 9     ; Co-Prozessor-Typ (0=kein, 4=486DX)
	DB      1               ; 10    ; Programmiersprache (1=ASM)
	DB      1               ; 11    ; Programm-Typ (0=System, 1=Anwender)
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DB      08h             ; 12    ; Flag (Bit 0 bis 7): (ja=0)
				;       ;   Bit 0-2: Task-Typ (NI-Task=000)
				;       ;   Bit 3: Task-Typ, Int-Nr. von Install?
				;       ;   Bit 4: Real-Mode Programm?
				;       ;   Bit 5: Caching von Code erlaubt?
				;       ;   Bit 6: - (=0 setzen)
				;       ;   Bit 7: Kein Hypertext im Programm?
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DB      4               ; 13    ; Flag (Bit 8 bis 15): (ja=0)
				;       ;   Bit 8: Daten von OsX reservieren?
				;       ;   Bit 9: Groesse Daten variabel?
				;       ;   Bit 10: Parameter von OsX reservieren?
				;       ;   Bit 11-15: reserviert
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DB      0               ; 14    ; Interrupt-Nr. (entfaellt, hier = 0)
	DB      0               ; 15    ; reserviert
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DD      0               ; 16    ; Anfangs-Adresse Datenbereich
	DD      0               ; 20    ; Groesse Datenbereich
	DD      0               ; 24    ; Minimum Datenbereich
	DD      10000h          ; 28    ; Maximum Datenbereich
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DW      OFFSET PAR0380  ; 32    ; Anfangs-Adresse Parameterbereich
	DW      0               ; 34    ;
	DW      PAR0380E - PAR0380      ; Groesse Parameterbereich
	DD      0               ; 38    ; Adresse Hypertextbereich
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DW      0               ; 42    ; reserviert
	DD      0               ; 44    ; reserviert fuer SORCUS
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DW      OFFSET M6P0380_M; 48    ; Adresse der Main-Prozedur (Nr.0)
	DW      0
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DW      OFFSET M6P0380_I; 52    ; Adresse der Auto-Init Prozedur (Nr.1)
	DW      0
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DW      OFFSET M6P0380_2; 56    ; Adresse Prozedur LED_ON (Nr.2)
	DW      0
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	DW      OFFSET M6P0380_3; 60    ; Adresse Prozedur LED_OFF (Nr.3)
	DW      0
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
M6P0380E:

;-----------------------------------------------------------------------------
;       Parameterbereich (ist Teil des Programms)
;-----------------------------------------------------------------------------
;       vor dem Parameterbereich muss Platz fuer die TDT freigehalten werden

	db      DEFLTDT dup (0)         ; Laenge TDT

;-----------------------------------------------------------------------------

PAR0380:                                ; Anfang Parameterbereich
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Name                  Init    rel.Ad.   Bedeutung
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
STATUS:         db      0       ; 0     ; Command/Status: 
					; Command: 0 = Stop 
					;          1 = Start Blinking
					; Status:  2 = Programm laeuft
ZULED:          db      0       ; 1     ; Zustand der LED: 0 = off, 1 = on
RATE:           dw      4000    ; 2     ; Blink-Rate (ungefaehr)
COUNTER:        dw      1       ; 4     ; Counter: auf 1, damit sofort umgeschaltet wird
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PAR0380E:                               ; Ende Parameterbereich

;-----------------------------------------------------------------------------
;       Main-Prozedur (= Prozedur 0): Blink LED
;-----------------------------------------------------------------------------
;       Vom Betriebssystem wird in dx bei Aufruf einer Prozedur (auch der 
;       Main-Prozedur) immer die Task-Nr uebergeben, unter der das Programm
;       installiert ist, zu dem die Prozedur gehoert. Das wird aber hier nicht 
;       weiter ausgewertet.

PROC M6P0380_M  FAR                             ; alle Prozeduren muessen FAR sein

		pusha                           ; es muessten nicht alle Register
		push    ds                      ;   gerettet werden, sondern nur die,
		push    es                      ;   die auch verwendet sind
		mov     ax,cs                   ; Tiny-Model: ds auf Anfang setzen
		mov     ds,ax
		xor     ax,ax                   ; es = 0 fuer Aufruf von Systemroutinen
		mov     es,ax
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		mov     al,[BYTE ds:STATUS]     ; Status = 2 (Run) ?
		cmp     al,2
		jnz     P0380_X                 ; nein

		mov     cx,[WORD ds:COUNTER]    ; Programm laeuft
		dec     cx
		jnz     P0380_LP                ; LED nicht umschalten

		mov     al,[BYTE ds:ZULED]      ; LED umschalten
		xor     al,1
P0380_ZX:       mov     [BYTE ds:ZULED],al      ; neuen Zustand der LED merken
		jz      P0380_OFF
		call    [DWORD es:LED_ON]
		jmp     P0380_LD
P0380_OFF:      call    [DWORD es:LED_OFF]

P0380_LD:       mov     cx,[WORD ds:RATE]       ; setze Zaehler neu
P0380_LP:       mov     [WORD ds:COUNTER],cx
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
P0380_END:      pop     es
		pop     ds
		popa
		ret                             ; daraus macht TASM "retf"

P0380_X:        cmp     al,3                    ; Status = 3 (Fehler) ?
		jae     P0380_END               ; ja, Ende
		and     al,al                   ; nein, Status = 0 (Programm Stop) ?
		jnz     P0380_MX
		mov     al,[BYTE ds:ZULED]      ; Setze LED auf gewuenschten Status
		cmp     al, 0
		jmp     P0380_ZX

P0380_MX:       mov     [BYTE ds:STATUS],2      ; setze Status immer = 2, also 
		jmp     P0380_END

ENDP    M6P0380_M

;-----------------------------------------------------------------------------
;       Auto-Init Prozedur (= Prozedur 1): Parameter auf Angangs-Zustand
;-----------------------------------------------------------------------------
;       Vom Betriebssystem wird in dx bei Aufruf einer Prozedur immer die 
;       Task-Nr uebergeben, das wird aber hier nicht ausgewertet.

PROC    M6P0380_I       FAR

		pusha
		push    ds
		mov     ax,cs                   ; Tiny-Model: ds auf Anfang setzen
		mov     ds,ax
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		mov     bx,OFFSET P0380_PI      ; Tabelle 
		mov     di,[WORD ds:bx]         ; erster zu initialisierender Parameter
		add     di,OFFSET PAR0380
		mov     cx,[WORD ds:bx+2]       ; Anzahl Byte
		add     bx,4                    ; Pointer
M6P0380_I1:     mov     al,[BYTE ds:bx]
		mov     [BYTE ds:di],al
		inc     di
		inc     bx
		dec     cx
		jnz     M6P0380_I1
P0380_I2:
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		pop     ds
		popa
		ret
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
P0380_PI:       dw      0                       ; erster zu initialisierender Parameter
		dw      6                       ; Anzahl (Byte) Parameter zu initialisieren

		db      0                       ; STATUS
		db      0                       ; ZULED: Led-Zustand
		dw      10000                   ; RATE: Blink-Rate
		dw      1                       ; COUNTER

ENDP    M6P0380_I

;-----------------------------------------------------------------------------
;       Prozedur 2: LED ON
;-----------------------------------------------------------------------------
;       Vom Betriebssystem wird in dx bei Aufruf einer Prozedur immer die 
;       Task-Nr uebergeben, das wird aber hier nicht ausgewertet.

PROC    M6P0380_2       FAR

		pusha
		push    ds
		push    es
		mov     ax,cs                   ; Tiny-Model: ds auf Anfang setzen
		mov     ds,ax
		xor     ax,ax
		mov     es,ax
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		call    [DWORD es:LED_ON]
		mov     al,1                    ; Zustand LED setzen
		mov     [BYTE ZULED],al         ; neuen Zustand der LED speichern
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		pop     es
		pop     ds
		popa
		ret

ENDP    M6P0380_2

;-----------------------------------------------------------------------------
;       Prozedur 3: LED OFF
;-----------------------------------------------------------------------------
;       Vom Betriebssystem wird in dx bei Aufruf einer Prozedur immer die 
;       Task-Nr uebergeben, das wird aber hier nicht ausgewertet.

PROC    M6P0380_3       FAR

		pusha
		push    ds
		push    es
		mov     ax,cs                   ; Tiny-Model: ds auf Anfang setzen
		mov     ds,ax
		xor     ax,ax
		mov     es,ax
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		call    [DWORD es:LED_OFF]
		mov     al,0                    ; Zustand LED setzen
		mov     [BYTE ZULED],al         ; neuen Zustand der LED speichern
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
		pop     es
		pop     ds
		popa
		ret

ENDP    M6P0380_3

;-----------------------------------------------------------------------------

	END M6P0380

;-------------------------- END OF M6P0380.ASM -------------------------------
