{***************************************************************************}
{**                 BLINK-LED WITH II-TASK (Multi-COM)                    **}
{**                                                                       **}
{**  This program shows how to use II-Tasks for Real-Time-Programs on     **}
{**  the Multi-COM board. It is a simple example to demonstrate the       **}
{**  usage of NI-Tasks (the on-board LED flashes with a programable       **}
{**  frequency). Timer-A is used to generate the blink-freqeuncy.         **}
{**  This program can also be installed as a NI-Task.                     **}
{**  For more details please refer to the Multi-COM Users Manual.         **}
{**                                                                       **}
{**  Installation-Reference:                                              **}
{**                                                                       **}
{**  program-number   : 301h                                              **}
{**  task-number      : defined by user                                   **}
{**  interrupt-source : 91h (Timer-A, II-Task) or 00h (none, NI-Task)     **}
{**  data-range       : 0                                                 **}
{**  flags            : 0989h (don't activate, call Auto-Init)            **}
{**                                                                       **}
{**                         for Turbo-Pascal 7.0                          **}
{**                                                                       **}
{**                             Version 1.A.                              **}
{**                                                                       **}
{**                               date : 02/04/98                         **}
{**                        last change : 02/04/98                         **}
{**                             status : released                         **}
{**                             author : A.H.                             **}
{**                                                                       **}
{**              (c)  1998 SORCUS Computer GmbH Heidelberg                **}
{***************************************************************************}
{$N-}                                  {* do not use coprocessor           *}
{$E-}                                  {* no emulation                     *}
{$R-}                                  {* no range checking                *}
{$S-}                                  {* no stack checking                *}
{$I-}                                  {* no IO checking                   *}
{$V-}                                  {* no string checking               *}
{$L+}                                  {* local symbols for Turbo-Debugger *}
{$F+}                                  {* force FAR calls!                 *}
{$M 1024, 0, 0}                        {* stack and heap to minimum        *}

USES ml6rtbib;                         {* include real time library        *}

				       {* miscellaneous                    *}
CONST
 PROG_NUMBER     : Word = $301;        {* program number = 301h            *}
 VERSION         : Char = '1';         {* program version                  *}
 REVISION        : Char = 'A';         {* program revision                 *}

				       {* program status:                  *}
 READY           = 0;                  {* ready                            *}
 RUNNING         = 2;                  {* runnning                         *}
 ERROR           = 3;                  {* error                            *}
 STOPPED         = 4;                  {* program stopped                  *}

				       {* status of LED                    *}
 ON              = 1;
 OFF             = 0;

VAR
   pdt         : RECORD                {* Program Descriptor Table (PDT)   *}
                  pdt_head : ML6_PDT_HEAD;
                  main_proc: POINTER;  {* address of main procedure        *}
	          auto_init: POINTER;  {* address of auto-init procedure   *}
                  start_proc: POINTER; {* address of start procedure       *}
                  stop_proc: POINTER;  {* address of stop procedure        *}
                 END;

				       {* parameter area definition        *}
   parameter   : RECORD
		  TDT       : TDT_TYPE;{* area for Task Descr. Table (TDT) *}
		  status    : Byte;    {* program status (READY, ...)      *}
		  blink_rate: Word;    {* blink rate                       *}
		  led_status: Byte;    {* LED status                       *}
		  soft_cnt  : Word;    {* soft counter                     *}
		 END;


PROCEDURE auto_init;                   {* AUTO-INIT PROCEDURE              *}
BEGIN
 ml6rt_entry;                          {* save registers                   *}

 parameter.status := READY;            {* set program-status = READY       *}
 parameter.blink_rate := 500;          {* set blink rate (default = 1 Hz)  *}
 parameter.led_status := OFF;          {* status of LED = OFF              *}
 ml6rt_local_led_off;                  {* turn off the on-board-LED        *}

 ml6rt_exit;                           {* restore registers                *}
END;


PROCEDURE start;
VAR timer : Word;
BEGIN

 ml6rt_entry;                          {* save registers                   *}

				       {* calc. timer value (1000us=1kHz)  *}
 timer := ml6rt_convert_timer_data(1000);
				       {* set Timer-A                      *}
 ml6rt_set_timer(TIMER_A, timer, INT_MODE);

 parameter.soft_cnt := parameter.blink_rate;
 parameter.status := RUNNING;

				       {* activate task                    *}
 ml6rt_wakeup_task(parameter.tdt.task);

 ml6rt_exit;                           {* restore registers                *}
END;


PROCEDURE stop;		               {* STOP PROCEDURE                   *}
BEGIN
 ml6rt_entry;                          {* save registers                   *}

 ml6rt_sleep_task(parameter.tdt.task); {* deactivate task                  *}
 parameter.status := STOPPED;          {* set status = STOPPED             *}

 ml6rt_exit;
END;                                   {* restore registers                *}


PROCEDURE main_task;                   {* MAIN PROCEDURE OF THE TASK       *}
BEGIN
 ml6rt_entry;                          {* save registers                   *}

 DEC(parameter.soft_cnt);              {* decrement soft counter           *}
 IF(parameter.soft_cnt = 0) THEN
   BEGIN                               {* if soft counter = 0, switch LED  *}
    IF(parameter.led_status = ON) THEN {* if the LED is on                 *}
      BEGIN
       ml6rt_local_led_off;            {* switch led off                   *}
       parameter.led_status := OFF;    {* ... save actual status           *}
      END
    ELSE                               {* if the LED is off                *}
      BEGIN
       ml6rt_local_led_on;
       parameter.led_status := ON;     {* ... save actual status           *}
      END;
				       {* re-initialize soft counter       *}
    parameter.soft_cnt := parameter.blink_rate;
   END;

 ml6rt_exit;                           {* restore registers                *}
END;


BEGIN                   	       {* PREPARE                          *}
				       {* initialize PDT                   *}
 pdt.pdt_head.ucType := 1;             {* set PDT type                     *}
				       {* set length of the PDT header     *}
 pdt.pdt_head.ucSize := sizeof(ML6_PDT_HEAD);
 pdt.pdt_head.usGlobalProc := 4;       {* number of global procedures      *}
 pdt.pdt_head.usProgNo  := PROG_NUMBER;{* program-number                   *}
 pdt.pdt_head.cVersion  := VERSION;    {* program-version                  *}
 pdt.pdt_head.cRevision := REVISION;   {* program-revision                 *}
 pdt.pdt_head.ucCPU     := _486SX;     {* type of CPU                      *}
 pdt.pdt_head.ucCoProc  := _NOCOPROZ;  {* type of coprocessor              *}
 pdt.pdt_head.ucLanguage := _CPP31;    {* programming language = C++       *}
 pdt.pdt_head.ucProgType := _USER_PRG; {* program-type: user application   *}
 pdt.pdt_head.usFlags    := 0;         {* flags, task type is defined      *}
 pdt.pdt_head.usFlags := _LOCAL_DATA + {* during installation!             *}
			_FIXED_DATASIZE +
			_LOCAL_PARAMETER;

 pdt.pdt_head.usInt := IRQ_TIMER_A;    {* interrupt-num. for the task      *}

 pdt.pdt_head.ulDataAdr  := 0;         {* address of data area             *}
 pdt.pdt_head.ulDataSize := 0;         {* size of data area                *}
 pdt.pdt_head.ulDataSizeMin := 0;      {* minimum size of data area        *}
 pdt.pdt_head.ulDataSizeMax := 0;      {* maximum size of data area        *}
				       {* start address of parameter area  *}
 ml6rt_phys_adr(@parameter, pdt.pdt_head.ulGlobalParAdr);
 pdt.pdt_head.ulGlobalParAdr := pdt.pdt_head.ulGlobalParAdr + sizeof(TDT_TYPE);
				       {* set number of parameters         *}
 pdt.pdt_head.usGlobalParSize := sizeof(parameter) - sizeof(TDT_TYPE);
 pdt.pdt_head.ulHyperAdr := 0;         {* address of hypertext area        *}
 pdt.main_proc  := @main_task;         {* address of main procedure        *}
 pdt.auto_init  := @auto_init;         {* address of auto-init procedure   *}
 pdt.start_proc := @start;             {* address of start procedure       *}
 pdt.stop_proc  := @stop;              {* address of stop procedure        *}

 ml6rt_set_pdt_adr(pdt);               {* set address of PDT for OsX       *}
END.


