{***************************************************************************}
{**                 BLINK-LED WITH NI-TASK (Multi-COM)                    **}
{**                       AND PC-SRQ-RELEASE                              **}
{**                                                                       **}
{**  This program shows how to use NI-Tasks for Real-Time-Programs on     **}
{**  the Multi-COM board. It is a simple example to demonstrate the       **}
{**  usage of NI-Tasks (the on-board LED flashes with a programable       **}
{**  frequency) and the release of service requests (SRQ) to the PC.      **}
{**  For more details please refer to the Multi-COM Users Manual.         **}
{**                                                                       **}
{**                                                                       **}
{**  Installation-Reference:                                              **}
{**                                                                       **}
{**  program-number   : 304h                                              **}
{**  task-number      : defined by user                                   **}
{**  interrupt-source : 00h (none, NI-Task)                               **}
{**  data-range       : 0                                                 **}
{**  flags            : 1980h (activate, call Auto-Init, NI-Task)         **}
{**                                                                       **}
{**                         for Turbo-Pascal 7.0                          **}
{**                                                                       **}
{**                             Version 1.A.                              **}
{**                                                                       **}
{**                               date : 02/04/98                         **}
{**                        last change : 02/04/98                         **}
{**                             status : released                         **}
{**                             author : A.H.                             **}
{**                                                                       **}
{**              (c)  1998 SORCUS Computer GmbH Heidelberg                **}
{**                                                                       **}
{***************************************************************************}
{$N-}                                  {* do not use coprocessor           *}
{$E-}                                  {* no emulation                     *}
{$R-}                                  {* no range checking                *}
{$S-}                                  {* no stack checking                *}
{$I-}                                  {* no IO checking                   *}
{$V-}                                  {* no string checking               *}
{$L+}                                  {* local symbols for Turbo-Debugger *}
{$F+}                                  {* force FAR calls!                 *}
{$M 1024, 0, 0}                        {* stack and heap to minimum        *}

USES ml6rtbib;                         {* include real time library        *}

				       {* miscellaneous                    *}
CONST
 PROG_NUMBER     : Word = $304;        {* program number = 304h            *}
 VERSION         : Char = '1';         {* program version                  *}
 REVISION        : Char = 'A';         {* program revision                 *}

				       {* program status:                  *}
 READY           = 0;                  {* ready                            *}
 START           = 1;                  {* start task                       *}
 RUNNING         = 2;                  {* runnning                         *}
 ERROR           = 3;                  {* error                            *}
 STOPPED         = 4;                  {* program stopped                  *}

				       {* status of LED                    *}
 ON              = 1;
 OFF             = 0;

VAR
   pdt         : RECORD                {* Program Descriptor Table (PDT)   *}
                  pdt_head : ML6_PDT_HEAD;
                  main_proc: POINTER;  {* address of main procedure        *}
		  auto_init: POINTER;  {* address of auto-init procedure   *}
		 END;

				       {* parameter area definition        *}
   parameter   : RECORD
		  TDT       : TDT_TYPE;{* area for Task Descr. Table (TDT) *}
		  status    : Byte;    {* program status (READY, ...)      *}
		  blink_rate: Word;    {* blink rate                       *}
		  led_status: Byte;    {* LED status                       *}
		  soft_cnt  : Word;    {* soft counter                     *}
		  countdown : Word;    {* number of runs ($FFFF=infinite)  *}
		  SRQ_enable: Boolean; {* enable SRQ (1=TRUE,0=FALSE)      *}
		  SRQ_sign  : Word;    {* request word to the pc           *}
		 END;


PROCEDURE auto_init;                   {* AUTO-INIT PROCEDURE              *}
BEGIN
 ml6rt_entry;                          {* save registers                   *}

 parameter.status := READY;            {* set program-status = READY       *}
 parameter.blink_rate := 50000;        {* set blink rate                   *}
 parameter.led_status := OFF;          {* status of LED = OFF              *}

 parameter.countdown  := $FFFF;        {* infinite running                 *}
 parameter.SRQ_enable := FALSE;        {* running without SRQ              *}
 parameter.SRQ_sign   := $0080;        {* SRQ word (lowbyte = $80..$C0)    *}

 ml6rt_local_led_off;                  {* turn off the on-board-LED        *}

 ml6rt_exit;                           {* restore registers                *}
END;



PROCEDURE main_task;                   {* MAIN PROCEDURE OF THE TASK       *}
BEGIN
 ml6rt_entry;                          {* save registers                   *}

 IF(parameter.status = START) THEN
   BEGIN
    parameter.status := RUNNING;
   END;

 IF((parameter.status = RUNNING) AND (parameter.countdown > 0)) THEN
   BEGIN
    DEC(parameter.soft_cnt);           {* decrement soft counter           *}
    IF(parameter.soft_cnt = 0) THEN
      BEGIN                            {* if soft counter = 0, switch LED  *}
       IF(parameter.led_status = ON) THEN
	 BEGIN                         {* if the LED is on                 *}
	  ml6rt_local_led_off;         {* switch led off                   *}
	  parameter.led_status := OFF; {* ... save actual status           *}
	 END
       ELSE                            {* if the LED is off                *}
	 BEGIN
	  ml6rt_local_led_on;
	  parameter.led_status := ON;  {* ... save actual status           *}

	  IF(parameter.SRQ_enable = TRUE) THEN
	    BEGIN
				       {* send SRQ to PC                   *}
	     ml6rt_send_buffer_srq(parameter.SRQ_sign, 1);
	    END;

	  IF(parameter.countdown <> $FFFF) THEN
            parameter.countdown:= parameter.countdown - 1;
	 END;
                                       {* re-initialize soft counter       *}
       parameter.soft_cnt := parameter.blink_rate;
      END;
   END;
 ml6rt_exit;                           {* restore registers                *}
END;


BEGIN                   	       {* PREPARE                          *}
				       {* initialize PDT                   *}
 pdt.pdt_head.ucType := 1;             {* set PDT type                     *}
				       {* set length of the PDT header     *}
 pdt.pdt_head.ucSize := sizeof(ML6_PDT_HEAD);
 pdt.pdt_head.usGlobalProc := 2;       {* number of global procedures      *}
 pdt.pdt_head.usProgNo  := PROG_NUMBER;{* program-number                   *}
 pdt.pdt_head.cVersion  := VERSION;    {* program-version                  *}
 pdt.pdt_head.cRevision := REVISION;   {* program-revision                 *}
 pdt.pdt_head.ucCPU     := _486SX;     {* type of CPU                      *}
 pdt.pdt_head.ucCoProc  := _NOCOPROC;  {* type of coprocessor              *}
 pdt.pdt_head.ucLanguage := _TP70;     {* programming language = Pascal    *}
 pdt.pdt_head.ucProgType := _USER_PRG; {* program-type: user application   *}
 pdt.pdt_head.usFlags    := 0;         {* flags                            *}
				       {* task type is defined by PDT      *}
 pdt.pdt_head.usFlags    := _NI_TASK + {* task type: NI-Task               *}
			   _PDT_INFO_ENABLE +
			   _LOCAL_DATA +
			   _LOCAL_PARAMETER +
			   _FIXED_DATASIZE;

 pdt.pdt_head.usInt := 0;              {* interrupt-num. for the task      *}
				       {* (not used with NI-Tasks          *}

 pdt.pdt_head.ulDataAdr  := 0;         {* address of data area             *}
 pdt.pdt_head.ulDataSize := 0;         {* size of data area                *}
 pdt.pdt_head.ulDataSizeMin := 0;      {* minimum size of data area        *}
 pdt.pdt_head.ulDataSizeMax := 0;      {* maximum size of data area        *}
				       {* start address of parameter area  *}
 ml6rt_phys_adr(@parameter, pdt.pdt_head.ulGlobalParAdr);
 pdt.pdt_head.ulGlobalParAdr := pdt.pdt_head.ulGlobalParAdr + sizeof(TDT_TYPE);
				       {* set number of parameters         *}
 pdt.pdt_head.usGlobalParSize := sizeof(parameter) - sizeof(TDT_TYPE);
 pdt.pdt_head.ulHyperAdr := 0;         {* address of hypertext area        *}
 pdt.main_proc  := @main_task;         {* address of main procedure        *}
 pdt.auto_init  := @auto_init;         {* address of auto-init procedure   *}

 ml6rt_set_pdt_adr(pdt);               {* set address of PDT for OsX       *}
END.


